<?php
/**
 * Copyright (C) 2019 Leipzig University Library
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License version 2 as
 * published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License along
 * with this program; if not, write to the Free Software Foundation, Inc.,
 * 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
 *
 * @author   Alexander Purr <purr@ub.uni-leipzig.de>
 * @license  http://opensource.org/licenses/gpl-2.0.php GNU GPLv2
 */

namespace fid_adlr\Controller;

use fid\Service\DataTransferObject\Order;
use fid\Validator\SubitoPartialCopyPageBounds;
use fid\Validator\SubitoPartialCopyPageSelection;
use Zend\Form\Form;
use Zend\Http\PhpEnvironment\Request;
use Zend\Session\Container;

class RecordController extends \fid\Controller\RecordController
{
    /**
     * @var Container
     */
    protected $session;

    /**
     * @param Container $session
     */
    public function setSession($session): void
    {
        $this->session = $session;
    }

    public function getOrderLabel($driver)
    {
        return $this->getRecordCitation($driver);
    }

    public function fidSubitoPartialCopyAction ()
    {
        if (!($user = $this->getUser()))
        {
            return $this->forceLogin();
        }

        $type = "subito-partial-copy";
        $driver = $this->loadRecord();
        $recordId = $driver->tryMethod('getUniqueID');
        $user = $this->client->requestUserDetails();
        $messenger = $this->flashMessenger();

        /** @var Form $form */
        /** @var Request $request */
        $request = $this->getRequest();
        $form = $this->serviceLocator->get('fid-acquisition-subito-partial-copy-form');

        // add number of pages specific validators
        $recordPages = $driver->tryMethod('getNumberOfPages');
        if ($recordPages)
        {
            $form->getInputFilter()->get('pages')->getValidatorChain()->attach(
                new SubitoPartialCopyPageBounds(['numberPages' => $recordPages]),
                true
            );

            // check if to pause selection validation for case that wrong warning message was thrown
            if (!(isset($this->session->wrongSelection) && $this->session->wrongSelection == $request->getPost()->get('pages')))
            {
                $form->getInputFilter()->get('pages')->getValidatorChain()->attach(
                    new SubitoPartialCopyPageSelection(['numberPages' => $recordPages])
                );
                $wasWarned = false;
            }
            else
            {
                $wasWarned = true;
            }
            unset($this->session->wrongSelection);
        }

        if ($submitted = $this->formWasSubmitted())
        {
            $form->setData($request->getPost());
            if ($form->isValid())
            {
                $url = $this->getRecordUrl($recordId);
                $label = $this->getOrderLabel($driver);
                $subitoPartialCopy = [
                    'selection' => $form->getData()['pages'],
                    'comment' => $form->getData()['comment'],
                    'warning' => $wasWarned
                ];
                $form->getHydrator()->hydrate(compact('driver','user', 'type', 'url', 'label', 'subitoPartialCopy'), $order = new Order());
                $this->client->requestOrderCreation($order);
                $message = $this->translate('fid::acquisition_success');
                $messenger->addSuccessMessage($message);
                return $this->redirect()->toRoute('record',['id' => $recordId]);
            }

            if ($errors = $form->getInputFilter()->get('pages')->getValidatorChain()->getMessages())
            {
                if( sizeof($errors) == 1 && array_key_exists ( SubitoPartialCopyPageSelection::PAGE_SELECTION, $errors ) )
                {
                    $this->session->wrongSelection = $request->getPost()->get('pages');
                }
            }
        }

        $action = $this->url()->fromRoute('record-fidsubitopartialcopy',['id' => $recordId]);
        $form->setAttribute('action', $action);
        $form->prepare();

        $view = $this->createViewModel();
        $view->setVariables(compact('form','driver','user'));
        $view->setTemplate('fid/record/acquisition-subito-partial-copy');

        return $view;
    }
}
